C
C   THIS PROGRAM CALCULATES THE SURFACE AREA OF A "FACETIZED SURFACE
C  MODEL" REPRESENTED BY A SURFMODL (.SRF) DATA FILE.  IT ALSO CHECKS
C  YOUR DATA FILE FOR FACETS FOR WHICH THE VERTICES DO NOT OBEY
C  SURFMODL'S RIGHTHAND RULE.  SEE BELOW FOR DETAILS ON HOW TO USE THIS
C  OPTION.  THE PROGRAM WAS WRITTEN FOR SURFMODL DATA FILE VERSIONS 1 -
C  4.  I'M SURE THE VERSION 4 OPTION WORKS OKAY AND I'M REASONABLY
C  CERTAIN ABOUT VERSION 3.  VERSIONS 1 AND 2, HOWEVER, I'M NOT SO SURE
C  ABOUT.  (I ONLY HAVE DOCUMENTATION FOR VERSION 4 FILES.)  THE PROGRAM
C  RUNS WITH THE LOWER VERSION FILES BUT YOU SHOULD CLOSELY INSPECT THE
C  CALCULATED SURFACE AREA FOR ANY OBVIOUS "WRONGNESS."  THE FIRST LINE
C  IN THE SURFMODL FILE MUST BE EITHER A TITLE OR COMMENT LINE.  COMMENT
C  LINES ARE ACCEPTABLE AT ONLY CERTAIN LOCATIONS IN THE .SRF FILE:  AT
C  THE VERY TOP OF THE FILE (BEFORE THE TITLE), BEFORE THE VERSION LINE,
C  BEFORE THE CONTROL LINE, BEFORE THE MATERIAL DATA LINES BEGIN, BEFORE
C  THE NODAL COORDINATE DATA LINES BEGIN, BEFORE THE SURFACE CONNECT-
C  IVITY DATA LINES BEGIN, OR AT THE VERY BOTTOM OF THE FILE (AFTER THE
C  CONNECTIVITY DATA, FOR WHATEVER REASON YOU'D WANT THEM THERE).  THE
C  PROGRAM DOES NOT CHECK FOR "ERRONEOUSLY" PLACED COMMENT LINES.  (IF
C  IT DID, THEIR PRESENCE WOULDN'T BE ERRONEOUS!)  A COMMENT LINE, BY
C  THE WAY, BEGINS WITH A "*" IN COLUMN 1.  (DON'T START WITH THE * IN
C  ANY OTHER COLUMN OR THE INPUT PROCESS WON'T WORK RIGHT.)  BLANK LINES
C  IN THE.SRF FILE ARE IGNORED.  AS A FURTHER, PERHAPS OBVIOUS, COMMENT,
C  IT IS ASSUMED THAT THE .SRF FILE IS A VALID SURFMODL DATA FILE.  (DO
C  NOT RUN THIS PROGRAM ON A FILE THAT SURFMODL THINKS IS ERRONEOUS.)
C
C   IF THE NAME OF THE SURFMODL FILE (WITH PATH) IS NOT SPECIFIED ON THE
C  DOS COMMAND-LINE (E.G., "SRFAREA C:\SURFMODL\ELLPSOID" or "SRFAREA
C  SPHERE.DAT"), THE PROGRAM PROMPTS YOU FOR IT.  IF YOU LEAVE OFF THE
C  EXTENSION, SRFAREA ASSUMES A .SRF EXTENSION.
C
C   THE PROGRAM ALSO OUTPUTS THE MAXIMUM X, Y, AND Z DIMENSIONS OF THE
C  SURFACE MODEL.
C
C   NOW, THEN, WHAT ABOUT THE ABOVE MENTIONED CHECK FOR VERTICES OBEYING
C  THE RIGHTHAND NORMAL RULE?  (THE SURFMODL DOCUMENT DEFINES THIS AS
C  THE NODES OF A FACET BEING ORDERED IN A COUNTERCLOCKWISE MANNER WHEN
C  VIEWED FROM THE OUTSIDE OF THE SURFACE.)  WHEN THIS OPTION IS
C  INVOKED, SRFAREA WILL COPY YOUR ORIGINAL FILE TO A NEW FILE.  IN THE
C  THE PROCESS, IT WILL REVERSE THE ORDERING OF THE VERTICES OF ANY
C  FACET (INDIVIDUAL SURFACE) THAT SRFAREA FINDS TO BE ORDERED WRONG.
C  THIS OPTION IS INVOKED IN TWO WAYS.  FIRST, YOU CAN SPECIFY THE NAME
C  OF A SECOND FILE ON THE DOS COMMAND-LINE.  (IF YOU DON'T GIVE AN
C  EXTENSION, .SRF IS ASSUMED.)  IF THIS SECOND PARAMETER IS BLANK,
C  SRFAREA WILL PROMPT YOU FOR A FILE NAME.  IF YOU THEN JUST TYPE
C  RETURN/ENTER, THIS FACET-CHECKING OPTION WILL BE SKIPPED.  OTHERWISE,
C  THIS IS THE FILE THAT THE CORRECTED DATA GETS WRITTEN TO.  SRFAREA
C  WILL ALSO WRITE INFORMATION TO THE SCREEN INFORMING YOU OF WHAT IT'S
C  DOING TO WHICH FACET.  IF IT FINDS NO FACETS "INCORRECTLY" DEFINED,
C  SRFAREA DELETES THIS NEW FILE BEFORE TERMINATING SINCE THERE IS NO
C  GOOD REASON TO HAVE CREATED IT IN THE FIRST PLACE.  (IF YOU JUST WANT
C  TO COPY A .SRF FILE, USE THE DOS COPY COMMAND!)  ALSO, IF YOUR OUTPUT
C  FILE IS THE SAME AS YOUR INPUT FILE, THE OPTION TO CHECK FOR DIS-
C  ORDERED VERTICES WILL BE SKIPPED.  FURTHER, YOU MAY WANT TO NOTE THAT
C  THE DETERMINATION OF COUNTERCLOCKWISE VERTEX ORDERING IS VERY
C  UNSOPHISTICATED.  IT ASSUMES THAT THE SURFACE IS A SIMPLE ONE ENCLOS-
C  ING THE COORDINATE ORIGIN.  YOU MAY VERY WELL HAVE A BOX, FOR
C  EXAMPLE, COMPLETELY DISPLACED FROM THE ORIGIN WITH VERTICES OBEYING
C  THE RIGHTHAND RULE.  BECAUSE IT USES A POSITION VECTOR FROM THE
C  ORIGIN TO DEFINE "OUTWARD," SRFAREA WILL SCREW UP THE VERTEX
C  ORDERING FOR SUCH A GEOMETRY.  (THE FAN2.SRF FILE THAT COMES WITH
C  SURFMODL IS AN EXAMPLE OF SUCH A FILE THAT SRFAREA CANNOT HANDLE
C  CORRECTLY IN THIS REGARD.)
C
      CHARACTER*80 DUMMY,TITLE
      CHARACTER*44 IFILE,OFILE
      CHARACTER*4 EXT
      CHARACTER*1 STAR
      INTEGER*2 ICON(9000),STATUS
      DIMENSION X(35000),Y(35000),Z(35000)
      CALL GETARG(1,IFILE,STATUS)
      IF(STATUS.NE.-1) GO TO 13
   14 CONTINUE
      WRITE(6,*)'SURFMODL INPUT FILE NAME?'
      READ(5,1)IFILE
      IF(IFILE.EQ.' ')GO TO 14
   13 CONTINUE
      CALL GETARG(2,OFILE,STATUS)
      IF(STATUS.NE.-1)GO TO 22
      WRITE(6,*)'NAME OF FILE TO STORE CORRECTED DATA?'
      READ(5,1)OFILE
   22 CONTINUE
      XMIN=1.E30
      YMIN=1.E30
      ZMIN=1.E30
      XMAX=-1.E30
      YMAX=-1.E30
      ZMAX=-1.E30
      EXT='.SRF'
      CALL NCREATE(IFILE,EXT)
      IF(OFILE.NE.' ')CALL NCREATE(OFILE,EXT)
      IF(OFILE.EQ.IFILE)OFILE=' '
      OPEN(1,STATUS='OLD',FILE=IFILE)
      IF(OFILE.NE.' ')OPEN(3,STATUS='UNKNOWN',FILE=OFILE)
C
C   THE STEPS BELOW ASSOCIATED WITH UNIT #2 ARE FOR THE PURPOSE OF
C  AVOIDING COMMENT STATEMENTS IN THE SURFMODL DATA FILE.
C
   21 CONTINUE
      READ(1,1)TITLE
      IF(OFILE.NE.' ')WRITE(3,6)TITLE
      READ(TITLE,1)STAR
      IF(STAR.EQ.'*')GO TO 21
   15 CONTINUE
      READ(1,1)DUMMY
      IF(OFILE.NE.' ')WRITE(3,6)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 15
      OPEN(2,STATUS='UNKNOWN',FILE='GARBAGE.FIL')
      WRITE(2,6)DUMMY
      REWIND 2
      READ(2,*)IVER
      CLOSE(2,STATUS='DELETE')
      IF(IVER.LT.1.OR.IVER.GT.4)CLOSE(1)
      IF(IVER.LT.1.OR.IVER.GT.4)STOP 'WRONG SURFMODL VERSION.'
   16 CONTINUE
      READ(1,1)DUMMY
      IF(OFILE.NE.' ')WRITE(3,6)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 16
      OPEN(2,STATUS='UNKNOWN',FILE='GARBAGE.FIL')
      WRITE(2,6)DUMMY
      REWIND 2
      IF(IVER.GE.3)READ(2,*)NMATL,NODES,NFACET,MAXVER,NSIDES
      IF(IVER.EQ.2)READ(2,*)NMATL,NODES,NFACET,IQ,MAXVER,NSIDES
      IF(IVER.EQ.1)READ(2,*)NMATL,NODES,MAXVER,NFACET
      CLOSE(2,STATUS='DELETE')
      IF(MAXVER.GT.9000.OR.NODES.GT.35000.OR.NFACET.GT.35000)CLOSE(1)
      IF(MAXVER.GT.9000)STOP 'CAN''T HANDLE MORE THAN 9000 NODES PER FAC
     .ET.'
      IF(NODES.GT.35000)STOP 'CAN''T HANDLE MORE THAN 35000 NODES.'
      IF(NFACET.GT.35000)STOP 'CAN''T HANDLE MORE THAN 35000 FACETS.'
   17 CONTINUE
      READ(1,1)DUMMY
      IF(OFILE.NE.' ')WRITE(3,6)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 17
      IF(NMATL.EQ.1)GO TO 18
      DO 3 I=1,NMATL-1
      READ(1,1)DUMMY
      IF(OFILE.NE.' ')WRITE(3,6)DUMMY
    3 CONTINUE
   18 CONTINUE
      READ(1,1)DUMMY
      IF(OFILE.NE.' ')WRITE(3,6)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 18
      OPEN(2,STATUS='UNKNOWN',FILE='GARBAGE.FIL')
      WRITE(2,6)DUMMY
      REWIND 2
      READ(2,*)X(1),Y(1),Z(1)
      CLOSE(2,STATUS='DELETE')
      IF(XMIN.GT.X(1))XMIN=X(1)
      IF(YMIN.GT.Y(1))YMIN=Y(1)
      IF(ZMIN.GT.Z(1))ZMIN=Z(1)
      IF(XMAX.LT.X(1))XMAX=X(1)
      IF(YMAX.LT.Y(1))YMAX=Y(1)
      IF(ZMAX.LT.Z(1))ZMAX=Z(1)
      DO 4 I=2,NODES
      READ(1,*)X(I),Y(I),Z(I)
      IF(OFILE.NE.' ')WRITE(3,*)X(I),Y(I),Z(I)
      IF(XMIN.GT.X(I))XMIN=X(I)
      IF(YMIN.GT.Y(I))YMIN=Y(I)
      IF(ZMIN.GT.Z(I))ZMIN=Z(I)
      IF(XMAX.LT.X(I))XMAX=X(I)
      IF(YMAX.LT.Y(I))YMAX=Y(I)
      IF(ZMAX.LT.Z(I))ZMAX=Z(I)
    4 CONTINUE
      ICOR=0
   19 CONTINUE
      READ(1,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 19
      FAREA=0.
      OPEN(2,STATUS='UNKNOWN',FILE='GARBAGE.FIL')
      WRITE(2,6)DUMMY
      REWIND 2
      READ(2,*)NVERT,MATER,(ICON(J),J=1,NVERT)
      CLOSE(2,STATUS='DELETE')
      CALL TRIANG(X(ICON(1)),Y(ICON(1)),Z(ICON(1)),X(ICON(2)),Y(ICON(2))
     *,Z(ICON(2)),X(ICON(3)),Y(ICON(3)),Z(ICON(3)),AREA,DL,DM,DN)
      FAREA=AREA+FAREA
      IF(OFILE.EQ.' ')GO TO 23
      DOTPRD=X(ICON(1))*DL+Y(ICON(1))*DM+Z(ICON(1))*DN
      IF(DOTPRD.LT.0)GO TO 24
      WRITE(3,*)NVERT,MATER,(ICON(J),J=1,NVERT)
      GO TO 23
   24 CONTINUE
      WRITE(6,*)'REVERSING VERTICES FOR FACET 1.'
      ICOR=ICOR+1
      WRITE(3,*)NVERT,MATER,(ICON(J),J=NVERT,1,-1)
   23 CONTINUE
      IF(NVERT.EQ.3)GO TO 8
      DO 9 L=3,NVERT-1
      CALL TRIANG(X(ICON(1)),Y(ICON(1)),Z(ICON(1)),X(ICON(L)),
     .Y(ICON(L)),Z(ICON(L)),X(ICON(L+1)),Y(ICON(L+1)),Z(ICON(L+1)),
     .AREA,DL,DM,DN)
      FAREA=AREA+FAREA
    9 CONTINUE
    8 CONTINUE
      IF(NFACET.EQ.1)GO TO 20
      DO 5 I=2,NFACET
      READ(1,*)NVERT,MATER,(ICON(J),J=1,NVERT)
      CALL TRIANG(X(ICON(1)),Y(ICON(1)),Z(ICON(1)),X(ICON(2)),Y(ICON(2))
     *,Z(ICON(2)),X(ICON(3)),Y(ICON(3)),Z(ICON(3)),AREA,DL,DM,DN)
      FAREA=AREA+FAREA
      IF(OFILE.EQ.' ')GO TO 25
      DOTPRD=X(ICON(1))*DL+Y(ICON(1))*DM+Z(ICON(1))*DN
      IF(DOTPRD.LT.0)GO TO 26
      WRITE(3,*)NVERT,MATER,(ICON(J),J=1,NVERT)
      GO TO 25
   26 CONTINUE
      WRITE(6,*)'REVERSING VERTICES FOR FACET ',I,'.'
      ICOR=ICOR+1
      WRITE(3,*)NVERT,MATER,(ICON(J),J=NVERT,1,-1)
   25 CONTINUE
      IF(NVERT.EQ.3)GO TO 5
      DO 10 L=3,NVERT-1
      CALL TRIANG(X(ICON(1)),Y(ICON(1)),Z(ICON(1)),X(ICON(L)),
     .Y(ICON(L)),Z(ICON(L)),X(ICON(L+1)),Y(ICON(L+1)),Z(ICON(L+1)),
     .AREA,DL,DM,DN)
      FAREA=AREA+FAREA
   10 CONTINUE
    5 CONTINUE
   20 CONTINUE
      CLOSE(1)
      IF(OFILE.NE.' '.AND.ICOR.EQ.0)CLOSE(3,STATUS='DELETE')
      IF(OFILE.NE.' '.AND.ICOR.NE.0)CLOSE(3)
      IF(OFILE.NE.' ')WRITE(6,*)' '
      IF(OFILE.NE.' ')WRITE(6,*)'REVERSED VERTICES ON ',ICOR,' FACETS.'
      WRITE(6,*)' '
      WRITE(6,*)'FACETIZED SURFACE AREA = ',FAREA
      WRITE(6,*)' '
      WRITE(6,*)'X-EXTENT = ',XMAX-XMIN
      WRITE(6,*)'Y-EXTENT = ',YMAX-YMIN
      WRITE(6,*)'Z-EXTENT = ',ZMAX-ZMIN
      STOP
    1 FORMAT(A)
    6 FORMAT(A80)
      END
C
C   THIS SUBROUTINE CONCATENATES A FILE NAME WITH AN EXTENSION TO CREATE
C  A COMPLETE DOS FILE NAME.  IF A "." IS DETECTED IN THE NAME VARIABLE,
C  NAME IS ASSUMED TO ALREADY CONTAIN AN EXTENSION AND IS THUS NOT MODI-
C  FIED.
C
      SUBROUTINE NCREATE(NAME,EXT)
      CHARACTER*44 DUMMY,NAME
      CHARACTER*4 EXT
      CHARACTER*1 B(40),A(40),C(4)
      READ(NAME,1)(A(I),I=1,40)
      J=0
      DO 2 I=1,40
      IF(A(I).EQ.' ')GO TO 2
      IF(A(I).EQ.'.')RETURN
      J=J+1
      B(J)=A(I)
    2 CONTINUE
    5 CONTINUE
      READ(EXT,1)(A(I),I=1,4)
      K=0
      DO 4 I=1,4
      IF(A(I).EQ.' ')GO TO 4
      K=K+1
      C(K)=A(I)
    4 CONTINUE
      WRITE(DUMMY,3)(B(I),I=1,J),(C(L),L=1,K)
      READ(DUMMY,1)NAME
      RETURN
    1 FORMAT(A)
    3 FORMAT(44A1)
      END
C
C   THIS SUBROUTINE CALCULATES THE AREA OF A TRIANGLE WITH VERTICES
C  (X1,Y1,Z1), (X2,Y2,Z2), AND (X3,Y3,Z3).  IN ADDITION, IT OUTPUTS
C  THE "RIGHTHAND" DIRECTION COSINES (L,M,N) OF ITS NORMAL VECTOR.
C
      SUBROUTINE TRIANG(X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3,AREA,L,M,N)
      REAL*4 L,M,N
C
C   SET DEFAULT VALUES FOR DIRECTION COSINES AND AREA IN CASE TRIANGLE
C  IS TOO THIN TO ACCURATELY EVALUATE.
C
      L=0.
      M=0.
      N=0.
      AREA=0.
C
C   GET LENGTHS OF SIDES AND CALCULATE AREA.
C
C   SINANG = SINE OF ANGLE BETWEEN BASE AND "SIDE 3."  (COSANG IS THE
C            COSINE OF THAT ANGLE.)
C
      BASE=SQRT((X1-X2)**2+(Y1-Y2)**2+(Z1-Z2)**2)
      S2=SQRT((X3-X2)**2+(Y3-Y2)**2+(Z3-Z2)**2)
      S3=SQRT((X3-X1)**2+(Y3-Y1)**2+(Z3-Z1)**2)
      IF(S3.LT.1.E-5.OR.BASE.LT.1.E-5)RETURN
      COSANG=(BASE**2+S3**2-S2**2)/2./BASE/S3
      SINANG=SQRT(1.-COSANG**2)
      HEIGHT=S3*SINANG
      AREA=HEIGHT*BASE/2.
      IF(SINANG.LT.1.E-5)RETURN
C
C   GET NORMAL VECTOR COMPONENTS (THE NORMALIZATION OF BASE VECTOR
C  CROSSED WITH SIDE 3 VECTOR).  ("S1" REFERS TO BASE.)
C
      S1XS3X=(Y2-Y1)*(Z3-Z1)-(Z2-Z1)*(Y3-Y1)
      S1XS3Y=(Z2-Z1)*(X3-X1)-(X2-X1)*(Z3-Z1)
      S1XS3Z=(X2-X1)*(Y3-Y1)-(Y2-Y1)*(X3-X1)
      L=S1XS3X/BASE/S3/SINANG
      M=S1XS3Y/BASE/S3/SINANG
      N=S1XS3Z/BASE/S3/SINANG
C
C   TAKE CARE OF TRUNCATION ERROR.
C
      VMAG=SQRT(L**2+M**2+N**2)
      L=L/VMAG
      M=M/VMAG
      N=N/VMAG
      RETURN
      END
