C
C   THIS PROGRAM COMBINES TWO SURFMODL (.SRF) DATA FILES TO FORM A
C  SINGLE COMPOSITE STRUCTURE.  THE NAMES OF THE INPUT AND OUTPUT FILES
C  CAN BE SUPPLIED ON THE DOS COMMAND-LINE WHEN SRFCOMB IS RUN OR THEY
C  WILL BE PROMPTED FOR.  IF AN EXTENSION IS NOT SUPPLIED WITH ANY FILE,
C  .SRF IS ASSUMED.
C
C   THE PROGRAM WAS WRITTEN FOR SURFMODL DATA FILE VERSIONS 1 - 4.  I'M
C  SURE THE VERSION 4 OPTION WORKS OKAY AND I'M REASONABLY CERTAIN ABOUT
C  VERSION 3.  VERSIONS 1 AND 2, HOWEVER, I'M NOT SO SURE ABOUT.  (I
C  ONLY HAVE DOCUMENTATION FOR VERSION 4 FILES.)  MOST IMPORTANT, WHAT-
C  EVER VERSION IT IS, IT MUST BE THE SAME FOR BOTH INPUT FILES!  THE
C  FIRST LINE IN EACH SURFMODL FILE MUST BE EITHER A TITLE OR COMMENT
C  LINE.  COMMENT LINES ARE ACCEPTABLE AT ONLY CERTAIN LOCATIONS IN THE
C  .SRF FILES:  AT THE VERY TOP OF THE FILE (BEFORE THE TITLE), BEFORE
C  THE VERSION LINE, BEFORE THE CONTROL LINE, BEFORE THE MATERIAL DATA
C  LINES BEGIN, BEFORE THE NODAL COORDINATE DATA LINES BEGIN, BEFORE THE
C  SURFACE CONNECTIVITY DATA LINES BEGIN, OR AT THE VERY BOTTOM OF THE
C  FILE (AFTER THE CONNECTIVITY DATA, FOR WHATEVER REASON YOU'D WANT
C  THEM THERE).  THE PROGRAM DOES NOT CHECK FOR "ERRONEOUSLY" PLACED
C  COMMENT LINES.  (IF IT DID, THEIR PRESENCE WOULDN'T BE ERRONEOUS!)  A
C  COMMENT LINE, BY THE WAY, BEGINS WITH A "*" IN COLUMN 1.  (DON'T
C  START WITH THE * IN ANY OTHER COLUMN OR THE INPUT PROCESS WON'T WORK
C  RIGHT.)  BLANK LINES IN THE.SRF FILES ARE IGNORED.  HOWEVER, SUCH
C  COMMENT LINES ARE NOT WRITTEN TO THE NEW FILE.  AS A FURTHER, PERHAPS
C  OBVIOUS, COMMENT, IT IS ASSUMED THAT THE .SRF FILES ARE VALID
C  SURFMODL DATA FILES.  (DO NOT RUN THIS PROGRAM ON FILES THAT SURFMODL
C  THINKS ARE ERRONEOUS.)
C
C   IT IS ASSUMED THAT COMBINING THE TWO GEOMETRIES IS A VALID THING TO
C  DO.  NO CONSISTENCY CHECKS OR OTHER "CONSTRAINTS FROM REALITY" ARE
C  MADE.  THE TOTAL NUMBER OF MATERIALS IN THE COMBINED FILE WILL BE THE
C  SUM OF THE TWO NUMBERS IN THE RESPECTIVE INPUT FILES.  NSIDES IN THE
C  COMBINED FILE WILL BE THE LARGER OF THE TWO ORIGINAL VALUES;
C  SIMILARLY FOR MAXVER.
C
      CHARACTER*80 DUMMY
      CHARACTER*44 FILE1,FILE2,OFILE
      CHARACTER*4 EXT
      INTEGER*2 IC(3000),STATUS
      DATA EXT/'.SRF'/
      CALL GETARG(1,FILE1,STATUS)
      IF(STATUS.NE.-1) GO TO 13
      WRITE(6,*)'FIRST SURFMODL INPUT FILE NAME?'
      READ(5,1)FILE1
   13 CONTINUE
      CALL GETARG(2,FILE2,STATUS)
      IF(STATUS.NE.-1)GO TO 22
      WRITE(6,*)'SECOND SURFMODL INPUT FILE NAME?'
      READ(5,1)FILE2
   22 CONTINUE
      CALL GETARG(3,OFILE,STATUS)
      IF(STATUS.NE.-1)GO TO 14
      WRITE(6,*)'OUTPUT FILE NAME?'
      READ(5,1)OFILE
   14 CONTINUE
      CALL NCREATE(FILE1,EXT)
      CALL NCREATE(FILE2,EXT)
      CALL NCREATE(OFILE,EXT)
      OPEN(1,STATUS='OLD',FILE=FILE1)
      OPEN(2,STATUS='OLD',FILE=FILE2)
      OPEN(3,STATUS='UNKNOWN',FILE=OFILE)
   21 CONTINUE
      READ(1,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*')GO TO 21
      WRITE(3,37)'COMBINATION OF TWO SURFMODL GEOMETRIES'
   16 CONTINUE
      READ(2,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 16
   15 CONTINUE
      READ(1,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 15
      WRITE(3,6)DUMMY
C
C   DUMMY SHOULD CURRENTLY CONTAIN IVER.  GET IT NUMERICALLY.
C
      OPEN(4)
      WRITE(4,6)DUMMY
      REWIND 4
      READ(4,*)IVER
      CLOSE(4)
   23 CONTINUE
      READ(2,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 23
C
C   MAKE SURE SECOND FILE IS SAME VERSION AS FIRST.
C
      OPEN(4)
      WRITE(4,6)DUMMY
      REWIND 4
      READ(4,*)IVER2
      CLOSE(4)
      IF(IVER2.NE.IVER)GO TO 24
   26 CONTINUE
      READ(1,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 26
      OPEN(4)
      WRITE(4,6)DUMMY
      REWIND 4
      IF(IVER.GE.3)READ(4,*)NMATL1,NODES1,NF1,MAXVR1,NSIDE1
      IF(IVER.EQ.2)READ(4,*)NMATL1,NODES1,NF1,IQ,MAXVR1,NSIDE1
      IF(IVER.EQ.1)READ(4,*)NMATL1,NODES1,MAXVR1,NF1
      CLOSE(4)
   27 CONTINUE
      READ(2,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 27
      OPEN(4)
      WRITE(4,6)DUMMY
      REWIND 4
      IF(IVER.GE.3)READ(4,*)NMATL2,NODES2,NF2,MAXVR2,NSIDE2
      IF(IVER.EQ.2)READ(4,*)NMATL2,NODES2,NF2,IQ,MAXVR2,NSIDE2
      IF(IVER.EQ.2)READ(4,*)NMATL2,NODES2,MAXVR2,NF2
      CLOSE(4)
C
C   GET COMBINED CONTROL AND MATERIAL DATA AND OUTPUT IT.
C
      NSIDES=NSIDE1
      IF(NSIDE2.GT.NSIDE1)NSIDES=NSIDE2
      MAXVER=MAXVR1
      IF(MAXVR2.GT.MAXVR1)MAXVER=MAXVR2
      NMATL=NMATL1+NMATL2
      IF(NMATL.GT.20)NMATL=20
      NFACET=NF1+NF2
      NODES=NODES1+NODES2
      IF(IVER.GE.3)WRITE(3,*)NMATL,NODES,NFACET,MAXVER,NSIDES
      IF(IVER.EQ.2)WRITE(3,*)NMATL,NODES,NFACET,IQ,MAXVER,NSIDES
      IF(IVER.EQ.2)WRITE(3,*)NMATL,NODES,MAXVER,NFACET
   17 CONTINUE
      READ(1,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 17
      WRITE(3,6)DUMMY
      IF(NMATL1.EQ.1)GO TO 18
      DO 3 I=1,NMATL1-1
      READ(1,1)DUMMY
      WRITE(3,6)DUMMY
    3 CONTINUE
   18 CONTINUE
      READ(2,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 18
      WRITE(3,6)DUMMY
      IF(NMATL2.EQ.1)GO TO 32
      DO 29 I=1,NMATL2-1
      READ(2,1)DUMMY
      WRITE(3,6)DUMMY
   29 CONTINUE
   32 CONTINUE
C
C   TRANSFER NODAL DATA FOR BOTH FILES.
C
      DO 30 J=1,2
   28 CONTINUE
      READ(J,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 28
      WRITE(3,6)DUMMY
      NODES=NODES1
      IF(J.EQ.2)NODES=NODES2
      DO 4 I=2,NODES
      READ(J,*)X,Y,Z
      WRITE(3,*)X,Y,Z
    4 CONTINUE
   30 CONTINUE
C
C   TRANSFER CONNECTIVITY DATA.
C
      DO 33 J=1,2
      NFACET=NF1
      IF(J.EQ.2)NFACET=NF2
      DO 31 I=1,NFACET
      IF(I.GT.1)GO TO 5
    7 CONTINUE
      READ(J,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 7
      OPEN(4)
      WRITE(4,6)DUMMY
      REWIND 4
      READ(4,*)NVER,NM,(IC(K),K=1,NVER)
      CLOSE(4)
      GO TO 34
    5 CONTINUE
      READ(J,*)NVER,NM,(IC(K),K=1,NVER)
   34 CONTINUE
      IF(J.EQ.1)GO TO 36
      DO 35 K=1,NVER
      IC(K)=IC(K)+NODES1
   35 CONTINUE
      NM=NM+NMATL1
      IF(NM.GT.20)NM=20
   36 CONTINUE
      WRITE(3,*)NVER,NM,(IC(K),K=1,NVER)
   31 CONTINUE
   33 CONTINUE
      GO TO 25
   24 CONTINUE
      WRITE(6,*)'BOTH INPUT FILES MUST BE OF SAME VERSION!'
   25 CONTINUE
      CLOSE(1)
      CLOSE(2)
      CLOSE(3)
      STOP
    1 FORMAT(A)
    6 FORMAT(A80)
   37 FORMAT(A50)
      END
      SUBROUTINE NCREATE(NAME,EXT)
      CHARACTER*44 DUMMY,NAME
      CHARACTER*4 EXT
      CHARACTER*1 B(40),A(40),C(4)
      READ(NAME,1)(A(I),I=1,40)
      J=0
      DO 2 I=1,40
      IF(A(I).EQ.' ')GO TO 2
      IF(A(I).EQ.'.')RETURN
      J=J+1
      B(J)=A(I)
    2 CONTINUE
      READ(EXT,1)(A(I),I=1,4)
      K=0
      DO 4 I=1,4
      IF(A(I).EQ.' ')GO TO 4
      K=K+1
      C(K)=A(I)
    4 CONTINUE
      WRITE(DUMMY,3)(B(I),I=1,J),(C(L),L=1,K)
      READ(DUMMY,1)NAME
      RETURN
    1 FORMAT(A)
    3 FORMAT(44A1)
      END
