C
C   THIS PROGRAM MAGNIFIES THE X, Y, AND Z DIMENSIONS OF A SURFMODL
C  (.SRF) DATA FILE BY CONSTANT FACTORS, I.E., IT DILATES THE VOLUME
C  OCCUPIED BY THE OBJECT.  BEFORE DILATION, IT WILL ALSO SHIFT EACH CO-
C  ORDINATE BY AN INPUT AMOUNT.  AFTER THESE TRANSFORMATIONS ARE PER-
C  FORMED, THE OBJECT IS ROTATED ABOUT EITHER THE X, Y, OR Z-AXES, AS
C  SPECIFIED BY NONZERO INPUT ROTATION ANGLES.  (ONLY ONE SUCH NONZERO
C  ANGLE CAN BE SPECIFIED.)  THE NAMES OF THE INPUT AND OUTPUT FILES
C  CAN BE SUPPLIED ON THE DOS COMMAND-LINE WHEN SRFTRN IS RUN OR THEY
C  WILL BE PROMPTED FOR.  IF AN EXTENSION IS NOT SUPPLIED WITH EITHER
C  FILE, .SRF IS ASSUMED.
C
C   THE PROGRAM WAS WRITTEN FOR SURFMODL DATA FILE VERSIONS 1 - 4.  I'M
C  SURE THE VERSION 4 OPTION WORKS OKAY AND I'M REASONABLY CERTAIN ABOUT
C  VERSION 3.  VERSIONS 1 AND 2, HOWEVER, I'M NOT SO SURE ABOUT.  (I
C  ONLY HAVE DOCUMENTATION FOR VERSION 4 FILES.)  THE FIRST LINE IN THE
C  SURFMODL FILE MUST BE EITHER A TITLE OR COMMENT LINE.  COMMENT LINES
C  ARE ACCEPTABLE AT ONLY CERTAIN LOCATIONS IN THE .SRF FILE:  AT THE
C  VERY TOP OF THE FILE (BEFORE THE TITLE), BEFORE THE VERSION LINE,
C  BEFORE THE CONTROL LINE, BEFORE THE MATERIAL DATA LINES BEGIN, BEFORE
C  THE NODAL COORDINATE DATA LINES BEGIN, BEFORE THE SURFACE CONNECT-
C  IVITY DATA LINES BEGIN, OR AT THE VERY BOTTOM OF THE FILE (AFTER THE
C  CONNECTIVITY DATA, FOR WHATEVER REASON YOU'D WANT THEM THERE).  THE
C  PROGRAM DOES NOT CHECK FOR "ERRONEOUSLY" PLACED COMMENT LINES.  (IF
C  IT DID, THEIR PRESENCE WOULDN'T BE ERRONEOUS!)  A COMMENT LINE, BY
C  THE WAY, BEGINS WITH A "*" IN COLUMN 1.  (DON'T START WITH THE * IN
C  ANY OTHER COLUMN OR THE INPUT PROCESS WON'T WORK RIGHT.)  BLANK LINES
C  IN THE.SRF FILE ARE IGNORED.  AS A FURTHER, PERHAPS OBVIOUS, COMMENT,
C  IT IS ASSUMED THAT THE .SRF FILE IS A VALID SURFMODL DATA FILE.  (DO
C  NOT RUN THIS PROGRAM ON A FILE THAT SURFMODL THINKS IS ERRONEOUS.)
C
      CHARACTER*80 DUMMY
      CHARACTER*44 IFILE,OFILE
      CHARACTER*4 EXT
      INTEGER*2 STATUS
      DATA EXT/'.SRF'/
      PI=4.*ATAN(1.)
      CALL GETARG(1,IFILE,STATUS)
      IF(STATUS.NE.-1) GO TO 13
      WRITE(6,*)'SURFMODL INPUT FILE NAME?'
      READ(5,1)IFILE
   13 CONTINUE
      CALL GETARG(2,OFILE,STATUS)
      IF(STATUS.NE.-1)GO TO 14
      WRITE(6,*)'OUTPUT FILE NAME?'
      READ(5,1)OFILE
   14 CONTINUE
      WRITE(6,*)'X, Y, AND Z DILATION FACTORS?'
      READ(5,*)XDIL,YDIL,ZDIL
      WRITE(6,*)'X, Y, AND Z SHIFT AMOUNTS?'
      READ(5,*)XS,YS,ZS
      XROT=0.
      YROT=0.
      ZROT=0.
      WRITE(6,*)'ROTATION ANGLE (DEGREES) ABOUT X-AXIS?'
      READ(5,*)XROT
      IF(XROT.NE.0.)GO TO 19
      WRITE(6,*)'ROTATION ANGLE (DEGREES) ABOUT Y-AXIS?'
      READ(5,*)YROT
      IF(YROT.NE.0.)GO TO 19
      WRITE(6,*)'ROTATION ANGLE (DEGREES) ABOUT Z-AXIS?'
      READ(5,*)ZROT
   19 CONTINUE
      IF(ABS(XROT).LT.1.E-5)XROT=0.
      IF(ABS(YROT).LT.1.E-5)YROT=0.
      IF(ABS(ZROT).LT.1.E-5)ZROT=0.
      XROT=XROT*PI/180.
      YROT=YROT*PI/180.
      ZROT=ZROT*PI/180.
      CALL NCREATE(IFILE,EXT)
      CALL NCREATE(OFILE,EXT)
      OPEN(1,STATUS='OLD',FILE=IFILE)
      OPEN(3,STATUS='UNKNOWN',FILE=OFILE)
C
C   THE STEPS BELOW ASSOCIATED WITH UNIT #2 ARE FOR THE PURPOSE OF
C  AVOIDING COMMENT STATEMENTS IN THE SURFMODL DATA FILE.
C
   21 CONTINUE
      READ(1,1)DUMMY
      WRITE(3,6)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*')GO TO 21
   15 CONTINUE
      READ(1,1)DUMMY
      WRITE(3,6)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 15
      OPEN(2,STATUS='UNKNOWN',FILE='GARBAGE.FIL')
      WRITE(2,6)DUMMY
      REWIND 2
      READ(2,*)IVER
      CLOSE(2,STATUS='DELETE')
   16 CONTINUE
      READ(1,1)DUMMY
      WRITE(3,6)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 16
      OPEN(2,STATUS='UNKNOWN',FILE='GARBAGE.FIL')
      WRITE(2,6)DUMMY
      REWIND 2
      IF(IVER.GE.3)READ(2,*)NMATL,NODES,NFACET,MAXVER,NSIDES
      IF(IVER.EQ.2)READ(2,*)NMATL,NODES,NFACET,IQ,MAXVER,NSIDES
      IF(IVER.EQ.1)READ(2,*)NMATL,NODES,MAXVER,NFACET
      CLOSE(2,STATUS='DELETE')
   17 CONTINUE
      READ(1,1)DUMMY
      WRITE(3,6)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 17
      IF(NMATL.EQ.1)GO TO 18
      DO 3 I=1,NMATL-1
      READ(1,1)DUMMY
      WRITE(3,6)DUMMY
    3 CONTINUE
   18 CONTINUE
      READ(1,1)DUMMY
      READ(DUMMY,1)STAR
      IF(STAR.EQ.'*'.OR.DUMMY.EQ.' ')GO TO 18
      OPEN(2,STATUS='UNKNOWN',FILE='GARBAGE.FIL')
      WRITE(2,6)DUMMY
      REWIND 2
      READ(2,*)X,Y,Z
      CLOSE(2,STATUS='DELETE')
      X=XDIL*(X+XS)
      Y=YDIL*(Y+YS)
      Z=ZDIL*(Z+ZS)
      CALL ROT(Y,Z,XROT)
      CALL ROT(Z,X,YROT)
      CALL ROT(X,Y,ZROT)
      WRITE(3,*)X,Y,Z
      DO 4 I=2,NODES
      READ(1,*)X,Y,Z
      X=XDIL*(X+XS)
      Y=YDIL*(Y+YS)
      Z=ZDIL*(Z+ZS)
      CALL ROT(Y,Z,XROT)
      CALL ROT(Z,X,YROT)
      CALL ROT(X,Y,ZROT)
      WRITE(3,*)X,Y,Z
    4 CONTINUE
    5 CONTINUE
      READ(1,1,END=7)DUMMY
      WRITE(3,6)DUMMY
      GO TO 5
    7 CONTINUE
      CLOSE(1)
      CLOSE(3)
      STOP
    1 FORMAT(A)
    6 FORMAT(A80)
      END
      SUBROUTINE NCREATE(NAME,EXT)
      CHARACTER*44 DUMMY,NAME
      CHARACTER*4 EXT
      CHARACTER*1 B(40),A(40),C(4)
      READ(NAME,1)(A(I),I=1,40)
      J=0
      DO 2 I=1,40
      IF(A(I).EQ.' ')GO TO 2
      IF(A(I).EQ.'.')RETURN
      J=J+1
      B(J)=A(I)
    2 CONTINUE
      READ(EXT,1)(A(I),I=1,4)
      K=0
      DO 4 I=1,4
      IF(A(I).EQ.' ')GO TO 4
      K=K+1
      C(K)=A(I)
    4 CONTINUE
      WRITE(DUMMY,3)(B(I),I=1,J),(C(L),L=1,K)
      READ(DUMMY,1)NAME
      RETURN
    1 FORMAT(A)
    3 FORMAT(44A1)
      END
      SUBROUTINE ROT(X,Y,ANGLE)
      TMP=X*COS(ANGLE)-Y*SIN(ANGLE)
      Y=X*SIN(ANGLE)+Y*COS(ANGLE)
      X=TMP
      RETURN
      END
